<?php
namespace Controller;

use Service\Traits\TResponse;
use Util\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DependencyInjection\InstanceManager;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Framework\DataSource\FilterCriteria;

class LogImportation {

    use TResponse;

	public function findNext(DTO\Request $request, DTO\Response $response)
	{
		try {
		    $params = $request->getParameters();
            $filter = Utilities::parseParamNextToZmartFilterCriteria('allOrganizations', $params);
            $limit = empty($filter->getPageSize()) ? 99999 : $filter->getPageSize();
            $conditions = $filter->getConditions();
            $this->applyDateRangeFilter($conditions, "LAST_IMPORTATION");

            //  Special filter conditions
            foreach ($conditions as &$condition) {
                if ($condition['columnName'] === "PAG") {
                    $condition['operator'] = "=";
                    $condition['value'] = intval($condition['value']);
                } else if ($condition['columnName'] === "TIME_IMPORTATION_SEG") {
                    $condition['operator'] = ">";
                    $condition['value'] = intval($condition['value']);
                }
            }

            /** @var \Service\LogImportation  $logImportation */
            $logImportation = InstanceManager::getInstance()->getService('\Service\LogImportation');
            $filter = $logImportation->rebuildFilterCriteriaWithAjustedConditions($filter, $conditions);
            //  Response
            $dataSet = Utilities::parseDataSourceZeedhiNext($logImportation->find($filter));
            //  Pagination
            $dataSet['pagination']['page'] = $filter->getPage();
            $dataSet['pagination']['limit'] = $limit;
            $dataSet['pagination']['total'] = $logImportation->count($conditions);
            $this->sendResponse($dataSet, 200);
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
	}

    public function findFailNext(DTO\Request $request, DTO\Response $response)
    {
        try {
            $params = $request->getParameters();
            $filter = Utilities::parseParamNextToZmartFilterCriteria('allOrganizations', $params);
            $limit = $filter->getPageSize();
            $conditions = $filter->getConditions();
            $this->applyDateRangeFilter($conditions, "dateLog");

            /** @var \Service\LogImportation  $logImportation */
            $logImportation = InstanceManager::getInstance()->getService('\Service\LogImportation');
            $filter = $logImportation->rebuildFilterCriteriaWithAjustedConditions($filter, $conditions);
            //  Response
            $dataResponse = Utilities::parseDataSourceZeedhiNext($logImportation->findFail($filter));
            //  Pagination
            $dataResponse['pagination']['page'] = $filter->getPage();
            $dataResponse['pagination']['limit'] = $limit;
            $dataResponse['pagination']['total'] = $logImportation->countFail($filter->getConditions());
            $this->sendResponse($dataResponse, 200);
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function applyDateRangeFilter(&$conditions, $columnName){
        foreach ($conditions as &$condition) {
            if ($condition['columnName'] === $columnName && is_array($condition['value'])) {
                $condition['operator'] = "BETWEEN";
            }
        }
    }

    public function configImportationFail(DTO\Request $request, DTO\Response $response)
    {
        try {
            $params = $request->getParameters();
            $filter = Utilities::parseParamNextToZmartFilterCriteria('logImportationFail', $params);
            $conditions = $filter->getConditions();
            /** @var \Service\NotificationFail  $logImportation */
            $logImportation = InstanceManager::getInstance()->getService('\Service\NotificationFail');
            $dataSet = Utilities::parseDataSourceZeedhiNext($logImportation->findFail($conditions));
            $dataSet = $this->setDataResponse($dataSet, $params);
            $this->sendResponse($dataSet, 200);
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function configImportationFailSave(DTO\Request\DataSet $request, DTO\Response $response)
    {
        try {
            $rows = $request->getDataSet()->getRows()[0];
            /** @var \Service\NotificationFail  $logImportation */
            $logImportation = InstanceManager::getInstance()->getService('\Service\NotificationFail');
            $saas = empty($rows['EMAIL_SAAS']) ? '' : $rows['EMAIL_SAAS'];
            $host = empty($rows['EMAIL_HOST']) ? '' : $rows['EMAIL_HOST'];
            $local = empty($rows['EMAIL_LOCAL']) ? '' : $rows['EMAIL_LOCAL'];
            $logImportation->saveConfigEmail($saas,$host,$local);
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

    public function notificationImportationFail(DTO\Request $request, DTO\Response $response)
    {
        try {
           // $conditions = $request->getFilterCriteria()->getConditions();
            /** @var \Service\LogImportation  $logImportation */
            $logImportation = InstanceManager::getInstance()->getService('\Service\LogImportation');
            $message =$logImportation->notificationImportationFail();
            $response->addMessage(new Message($message));

        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
    }

	public function importLogImportation(DTO\Request\Filter $request, DTO\Response $response)
	{
		try {
            /** @var \Service\LogImportation  $logImportation */
            $logImportation = InstanceManager::getInstance()->getService('\Service\LogImportation');
            $logImportation->importLogImportation();
        } catch(\Exception $ex){
            $response->setError(new Response\Error($ex->getMessage(), $ex->getCode()));
        }
	}


}
