<?php
namespace Controller;

use Service\Traits\TResponse;
use Service\ColumnsQueryService;
use Service\ConnectionService;
use Service\DataSourceService;
use Service\DeleteCascadeService;
use Service\Manager\Mongo\Manager;
use Util\Utilities;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Zmart\DataSource\ZmartFilterCriteria;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO;

class StructureType extends Crud{
    
    use TResponse;

    const ERRORS_CONNECT_FAIL = "PHP Error occurred 'oci_connect()";
    const ERRORS_EXECUTE_QUERY_FAIL = "PHP Error occurred 'oci_execute()";

    const RESPONSE_STRING_CONNECTION_NOT_FOUND = "Connection not found: ";
    const RESPONSE_STRING_FAIL_AT_CREATE = "Wasn't possible create the connection: ";
    const RESPONSE_STRING_CONNECTION_FAIL = "Wasn't possible connect with database, the old columns were kept.";
    const RESPONSE_STRING_EXECUTE_QUERY_FAIL = "Wasn't possible sync your columns. Fail to execute query: ";

    protected $dataSourceName = 'structureType';

    /** @var Connection */
    protected $connection;
    
    protected $dataSourceService;
    
    protected $dataSourceOtherService;
    
    protected $socketClient;
    
    protected $environment;

    /** @var ConnectionService */
    protected $connectionService;

    public function __construct(Manager $organizationMongoManager, DeleteCascadeService $deleteCascadeService, ConnectionService $connectionService, \Service\DataSourceService $dataSourceService, \Service\DataSource $dataSourceOtherService, \Service\SocketClient $socketClient, \Util\Environment $environment) {
        $organizationMongoManager->createDatabase(\Util\Utilities::getOrganizationIdForMongo());
        $this->connectionService = $connectionService;
        $this->socketClient = $socketClient;
        $this->dataSourceService = $dataSourceService;
        $this->dataSourceOtherService = $dataSourceOtherService;
        $this->environment = $environment;
        parent::__construct($organizationMongoManager, $deleteCascadeService);
    }

    public function findNext(DTO\Request $request, DTO\Response $response) {
        $params = $request->getParameters();
        //	parent::find($request, $response);
        $filter = new ZmartFilterCriteria($this->dataSourceName, array());
        
        // Order
        if (array_key_exists("order", $params)) {
            foreach ($params["order"] as $order) {
                $orderSplit = explode(".", $order);
                $filter->addOrderBy($orderSplit[0], $orderSplit[1]==="desc"? FilterCriteria::ORDER_DESC : FilterCriteria::ORDER_ASC);
            }
        }
        // Search ALL
        if (array_key_exists("search_in", $params)) {
            $searchValues = $params["search_in"];
            $filter->addCondition(implode('|',$searchValues), FilterCriteria::LIKE_ALL, "%".$params["search"]."%");
        }
        // Filter by field
        if (array_key_exists("in", $params)) {
            foreach ($params["in"] as $paramFilter){
                if(!array_key_exists($paramFilter, $params))
                    continue;
                $operator = FilterCriteria::EQ;
                $filterValue = $params[$paramFilter];
                if (is_string($filterValue)){
                    $filter->addCondition($paramFilter, FilterCriteria::LIKE_I, $filterValue);
                }
                else if (is_array($filterValue) && count($filterValue) > 0){
                    foreach ($filterValue as $value)
						$filter->addCondition($paramFilter, FilterCriteria::IN, $value);
                }
                else {
                    $filter->addCondition($paramFilter, $filterValue);
                }
            }
        }
        
        $dataSet = $this->mongoManager->findBy($filter);
        $response->addDataSet($dataSet);
        $dataSet = $response->getDataSets()[0];
        $structureTypes = $dataSet->getRows();
        $connections = $this->getConnections($structureTypes)->getRows();
        $indexedConnections = $this->indexByMongoId($connections);
        $hierarchyTypes = $this->relateStructureTypeWithConnection($structureTypes, $indexedConnections);
        
        $data = $hierarchyTypes;
        $total = count($data);
        $limit = isset($params["limit"]) ? (int)$params["limit"] : 9999;
        $page = isset($params["page"]) ? (int)$params["page"] : 1;
        $data = array_chunk($data, $limit);
        if ($page > count($data))
            $page = count($data);
        if ($page <= 0)
            $page = 1;
        if (count($data) > 0)
            $data = $data[$page-1];
        
        $this->sendResponse(["data" => array(
	        "structureType" => $data,
	        "pagination" => ["total" => $total, "page" => $page, "limit" => $limit]
        )], 200);
        //  $newDataSet = new DataSet($this->dataSourceName, $hierarchyTypes);
        //  $response->setDataSets(array($newDataSet));
    }

    protected function getConnections ($structureTypes) {
        $usedIds = $this->getConnectionsIdsUsed($structureTypes);
        $numberOfExpectedLines = count($usedIds);
        $filterCriteria = new FilterCriteria('connection', array(), null, $numberOfExpectedLines);
        $filterCriteria->addCondition('_id', FilterCriteria::IN, $usedIds);
        return $this->mongoManager->findBy($filterCriteria);
    }

    protected function getConnectionsIdsUsed ($structureTypes) {
        //For not repeat Ids used more than one time
        $connectionIdsIndexedById = $this->getDistinctConnectionsIds($structureTypes);
        //Build distinct ids in a index array
        $distictConnectionIds = array();
        foreach($connectionIdsIndexedById as $connectionId) {
            $distictConnectionIds[] = $connectionId;
        }
        return $distictConnectionIds;
    }

    protected function getDistinctConnectionsIds($structureTypes) {
        $connectionsIdsIndexedById = array();
        foreach($structureTypes as $structureType) {
            if (isset($structureType['connection'])) {
                $connectionId = $structureType['connection'];
                $connectionsIdsIndexedById[$connectionId] = $connectionId;
            }
        }
        return $connectionsIdsIndexedById;
    }

    protected function indexByMongoId($rows) {
        $newRows = array();
        foreach($rows as $row) {
            $newRows[$row['_id']] = $row;
        }
        return $newRows;
    }

    protected function relateStructureTypeWithConnection($structureTypes, $indexedByIdConnections) {
        foreach($structureTypes as &$structureType) {
            if (isset($structureType['connection']) && isset($indexedByIdConnections[$structureType['connection']])) {
                $structureType['connectionName'] = $indexedByIdConnections[$structureType['connection']]['name'];
            }
        }
        return $structureTypes;
    }

    public function getFieldKeysFromQuery(DTO\Request $request, DTO\Response $response) {
        $request = $this->buildDataSetIfRow($request);
        $row = $request->getDataSet()->getRows()[0];
        try {
			//TODO: use custom connection if there is one (first it need to be implemented in the websocket server)
            $columns = $this->getColumnsFromWebSocket($row);
            $response->addDataSet(new DataSet('columns', $columns));
        } catch (Exception $e) {
            if($e->getCode() === Exception::CODE_FAIL_CONNECTION || $e->getCode() === Exception::CODE_QUERY_EXECUTION_FAIL) {
                $response->addMessage(new Message($e->getMessage(), Message::TYPE_ERROR));
            } else {
                throw $e;
            }
        }
    }
    
    protected function getColumnsFromWebSocket($row) {
        try {
            $cleanedQuery = $this->dataSourceOtherService->replaceQueryParameters($row['query']);
            $organizationID = Utilities::getOrganizationIdForMongo();
            return $this->dataSourceService->getMergedColumnsFromWebSocket($this->socketClient, $cleanedQuery, $row, $organizationID);
        } catch (\Zeedhi\Framework\ErrorHandler\ErrorException $e) {
            if (strrpos($e->getMessage(), self::ERRORS_CONNECT_FAIL)) {
                throw Exception::connectFail();
            }
            else if (strrpos($e->getMessage(), self::ERRORS_EXECUTE_QUERY_FAIL)) {
                throw Exception::executeQueryError($row['query'], $e);
            }
            else {
                throw $e;
            }
        }
    }
}