<?php

namespace Service;

use Zeedhi\Framework\DataSource\FilterCriteria;
use \Service\Manager\Mongo\Manager;
use Util\Utilities;
use \Service\DataSource;
use Service\ConnectionService;
use Util\InstanceProvider;

class Structure {

    const CASHIER = "C";
    const UNIT = "F";
    const SUPERVISOR = "S";
    const REGIONAL = "R";

	protected $entityManager;
	protected $util;
    protected $dataSourceName = "structure";
    /** @var \Service\Manager\Mongo\Manager $organizationManager */
    protected $organizationManager;

	public function __construct(\Doctrine\ORM\EntityManager $entityManager, Manager $organizationManager, DataSource $dataSourceService, ConnectionService $connectionService)  {
		$this->entityManager = $entityManager;
        $this->organizationManager = $organizationManager;
        $this->dataSourceService = $dataSourceService;
        $this->connectionService = $connectionService;
    }

    public function getOrgManagerCdoperador(){
        $organizationId = Utilities::getOrganizationIdForMongo();
        $query = "SELECT OPE.CDOPERADOR FROM ORGANIZACAO ORG JOIN OPERADOR OPE ON OPE.DSEMAILOPER = ORG.DSEMAILOPER WHERE ORG.NRORG = $organizationId";
        $entityManager = InstanceProvider::getConnectionCustom()->updateVpdConnection($organizationId,true);
        $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();
        $rows = $entityManager->getConnection()->fetchAll($query);
        if(!empty($rows) && !empty($rows[0]) && !empty($rows[0]["CDOPERADOR"])){
            return $rows[0]["CDOPERADOR"];
        }
        return null;
    }

    public function saveLogStructure($dataBefore,$dataAfter,$id,$name,$type){
        Utilities::setLog(
            $dataBefore,
            $dataAfter,
            $this->dataSourceName,
            $id ?: null,
            "_".$name,
            null,
            array("structureType" => $type)
        );
    }
    
    public function loadStructuresFromQuery($structureType,$oldStructures=[]){
        //TODO: use custom connection if there is one (first it need to be implemented in the websocket server)
        $query = $structureType['query'];
        $organizationId = Utilities::getOrganizationIdForMongo();
        $clientRoom = $this->connectionService->getClientRoom($organizationId, $structureType);
        $connectionId = $structureType["connection"] ?: null;
        $rows = $this->dataSourceService->execQueryFromWebSocket($query, new FilterCriteria($this->dataSourceName),
            $clientRoom, 7, $connectionId,"","",false);
        $fieldKey = $structureType['fieldKey'];
        $descriptionField = $structureType['descriptionField'];

        $bdStructures = array();
        foreach ($rows as $row) {
            $bdStructures[] = $this->generateStructureFromQueryData($row, $fieldKey, $descriptionField, $structureType['_id']);
        }

        $newStructures = $this->getNewStructures($bdStructures, $oldStructures);
        foreach($newStructures as &$newStructure) {
            $newStructure['externalIdString'] = Utilities::encodeStructureExternalId($newStructure['externalId']);
        }
        return $newStructures;
    }

    public function generateStructureFromQueryData($row, $fieldKeys, $descriptionField, $type) {
        $structure = array(
            "name" => $row[$descriptionField],
            "externalId" => "",
            "usersResponsible" => null,
            "type" => $type
        );
        $externalId = array();
        foreach ($fieldKeys as $fieldKey) {
            $externalId[$fieldKey] = $row[$fieldKey];
        }
        $structure['externalId'] = $externalId;
        return $structure;
    }

    public function getNewStructures($bdStructures, $oldStructures) {
        $indexedBdStructures = $this->indexStructuresByExternalId($bdStructures);
        $indexedNewStructures = $this->unsetOldStructures($indexedBdStructures, $oldStructures);
        $newStructures = array();
        foreach ($indexedNewStructures as $structure) {
            $newStructures[] = $structure;
        }
        return $newStructures;
    }

    protected function indexStructuresByExternalId($structures) {
        $indexedStructures = array();
        foreach ($structures as $structure) {
            $indexedStructures[Utilities::encodeStructureExternalId($structure['externalId'])] = $structure;
        }
        return $indexedStructures;
    }

    protected function unsetOldStructures($indexedStructures, $oldStructures) {
        foreach ($oldStructures as $oldStructure) {
            $oldExternalId = Utilities::encodeStructureExternalId($oldStructure['externalId']);
            if($this->hasStructure($indexedStructures, $oldExternalId)) {
                unset($indexedStructures[$oldExternalId]);
            }
        }
        return $indexedStructures;
    }

    protected function hasStructure($indexedStructure, $key) {
        return isset($indexedStructure[$key]);
    }

    public function findBy(FilterCriteria $filterCriteria) {
        return $this->organizationManager->findBy($filterCriteria)->getRows();
    }

    public function findById($id) {
        $filterCriteria = new FilterCriteria($this->dataSourceName, array(array("columnName"=>"_id", "value" => $id, "operator" => "=")), null, 10000);
        $structure = $this->organizationManager->findBy($filterCriteria)->getRows();
        return count($structure) > 0 ? $structure[0] : null;
    }
    
    /**
     * $id shold be an Array 
     **/
    public function findByIds($id) {
        $filterCriteria = new FilterCriteria($this->dataSourceName, array(
            array("columnName"=>"_id", "value" => $id, "operator" => "IN")
        ), null, 10000);
        $structures = $this->organizationManager->findBy($filterCriteria)->getRows();
        return count($structures) > 0 ? $structures : null;
    }

    public function findAll() {
        return $this->organizationManager->findBy(new FilterCriteria($this->dataSourceName, array(), null, 10000))->getRows();
    }

    public function findByStructureTypes($structureTypes) {
        return $this->organizationManager->findBy(new FilterCriteria($this->dataSourceName, array(
            array("columnName"=>"type", "value" => $structureTypes, "operator" => "IN")), null, 10000)
        )->getRows();
    }

    public function findStructureTypeById($id) {
        $filterCriteria = new FilterCriteria('structureType', array(array("columnName"=>"_id", "value" => $id, "operator" => "=")), null, 10000);
        $structureType = $this->organizationManager->findBy($filterCriteria)->getRows();
        return count($structureType) > 0 ? $structureType[0] : null;
    }

    public function findAllStructureTypes() {
        $filterCriteria = new FilterCriteria('structureType', array(), null, 10000);
        $structureType = $this->organizationManager->findBy($filterCriteria)->getRows();
        return $structureType;
    }

    public function getChildStructures($structureId) {
        $childStructures = array();
        $filterCriteria = new FilterCriteria('hierarchy', array(), null, 10000);
        $filterCriteria->addCondition('structureParent', FilterCriteria::EQ, $structureId);
        $hierarchies = $this->organizationManager->findBy($filterCriteria)->getRows();

        if (!empty($hierarchies)) {
            $childStructure = $this->findById($hierarchies[0][Manager::MONGO_KEY_COLUMN]);
            $childStructures[] = $childStructure;
        }

        return $childStructures;
    }
}