<?php

namespace Service;


use Util\Utilities;
use \Teknisa\Model\Geral\Entities;
use Doctrine\ORM\EntityManager;
use Util\InstanceProvider;

class ZmartOperator {
    public function getNrOrgFilter() {
        return Utilities::getOrganizationIdForFindAcl();
    }

    public function getNrorgoper(){
        return Utilities::getNrorgoper();
    }

    public function find($conditions, $page, $pageSize)
    {
        $branches = null;
        foreach ($conditions as $key => $condition) {
            if($condition['columnName'] === 'CDFILIAL'){
                $branches = "'".implode("','", $condition['value'])."'";
                unset($conditions[$key]);
            }
        }
        $filters = Utilities::createQueryFiltered($conditions);
        if(!is_null($branches)){
            $filters .= $this->buildFilialFilter($branches);
        }
        $organizationId = $this->getNrOrgFilter();
        $nrorgoper = $this->getNrorgoper();
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        return $this->findOperators($organizationId, $nrorgoper, $filters, $limit, $offset);
    }

    private function buildFilialFilter($branches){
        return " AND (CDOPERADOR IN (SELECT CDOPERADOR FROM FILIOPER WHERE CDFILIAL IN($branches)))";
    }

    public function findOperators($organizationId, $nrorgoper, $filters, $limit, $offset)
    {
        $filterNroegoper = "AND NVL ( NRORGOPER, NRORG ) = $organizationId";
        if($nrorgoper && $nrorgoper != 20 && $organizationId != $nrorgoper){
            $filterNroegoper = "AND NRORGOPER = $nrorgoper";
        }
        $query = <<<SQL
            SELECT *
              FROM (SELECT PARC.NRINSCRICAOPARC, PARC.NMPRINCIPALPARC AS NMPARCNEGOCIO, OPER.NMACESSO, OPER.CDOPERADOR,
                           OPER.CDPAIS, OPER.NMOPERADOR, OPER.DTULTATUOPER, OPER.CDCLIENTE, OPER.CDEMPRESA, OPER.DSEMAILOPER, OPER.IDOPERMASTER,
                           OPER.IDOPERACESSO, OPER.IDOPERATIVO, OPER.NRORG, OPER.DTINCLUSAO, OPER.DTULTATU,
                           OPER.NRORGINCLUSAO, OPER.CDOPERINCLUSAO, OPER.CDOPERULTATU, OPER.NRORGULTATU,
                           OPER.NRPARCNEGOCIO, OPER.NRORGSELECIONADA, OPER.CDFILIALSELECIONADA, OPER.NRPERFILSELECIONADO,
                           OPER.NRORGTRAB, OPER.NRESTRUTTRAB, OPER.IDATIVO, OPER.IDTPOPERADOR, ORG.NMORGANIZACAO, EST.NMESTRUTURAM,
                           CLI.NMRAZSOCCLIE, PA.NMPAIS, OPER.DTULTACESOPER, OPER.NRORGOPER
                      FROM OPERADOR OPER
                 LEFT JOIN ORGANIZACAO ORG ON ORG.NRORG = OPER.NRORG
                 LEFT JOIN ESTRUTURAM EST ON EST.NRORG = OPER.NRORG
                                         AND EST.NRESTRUTURAM = OPER.NRESTRUTTRAB
                 LEFT JOIN CLIENTE CLI ON CLI.NRORG = OPER.NRORG
                                      AND CLI.CDCLIENTE = OPER.CDCLIENTE
                 LEFT JOIN PAIS PA ON PA.CDPAIS = OPER.CDPAIS
                 LEFT JOIN PARCNEGOCIO PARC ON PARC.NRORG = OPER.NRORG
                                           AND PARC.NRPARCNEGOCIO = OPER.NRPARCNEGOCIO)
             WHERE 1 = 1
             $filterNroegoper
             $filters

SQL;
        $queryWithPagination = Utilities::getPaginatedQuery($query, $limit, $offset);
        $entityManager = InstanceProvider::getConnectionCustom()->updateVpdConnection($organizationId,true);
        $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();
        return $entityManager->getConnection()->fetchAll($queryWithPagination);
    }

    public function saveOperatorEmail($cdOperador, $dsemailoper, $organizationId, $cdOperadorAtualizacao, $nrorgAtualizacao){
        $query = <<<SQL
        UPDATE OPERADOR SET
        DSEMAILOPER = :DSEMAILOPER,
        DTULTATU = CURRENT_TIMESTAMP,
        NRORGULTATU = :NRORGULTATU,
        CDOPERULTATU = :CDOPERULTATU
        WHERE CDOPERADOR = :CDOPERADOR
SQL;
        $param = array(
            'CDOPERADOR' => $cdOperador,
            'DSEMAILOPER' => $dsemailoper,
            'CDOPERULTATU' => $cdOperadorAtualizacao,
            'NRORGULTATU' => intval($nrorgAtualizacao)
        );
        $entityManager = InstanceProvider::getConnectionCustom()->updateVpdConnection($organizationId,true);
        $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();
        try{
            $result = $entityManager->getConnection()->executeUpdate($query,$param);
        }catch(\Exception $e){
            throw $e;
        }
        return $result;
    }

    public function findByNrorg($organizationId, $conditions, $page, $pageSize, $orderBy){
        try{
            $limit = $page * $pageSize;
            $offset = ($page - 1) * $pageSize;
            $filter = Utilities::createQueryFiltered($conditions);
            $orderByString = Utilities::getStringOrderBy($orderBy);
            $query = <<<SQL
            SELECT * FROM (
                SELECT O.CDOPERADOR, O.NMOPERADOR, O.DSEMAILOPER
                FROM OPERADOR O
                WHERE O.NRORG = :NRORG
            ) WHERE 1=1
            $filter
            $orderByString
SQL;
            $param = array(
                'NRORG' => $organizationId
            );
            $query = Utilities::getPaginatedQuery($query, $limit, $offset);
            $entityManager = InstanceProvider::getConnectionCustom()->updateVpdConnection($organizationId,true);
            $entityManager = $entityManager ? $entityManager : InstanceProvider::getEntityManager();

            $result = $entityManager->getConnection()->fetchAll($query,$param);
        }catch(\Exception $e){
            throw $e;
        }
        return $result;
    }
}
